{*******************************************************************}
{*                           SWRANDOM.PAS                          *}
{*               Some useful things with random numbers            *}
{*        Most of them are simly Turbo Pascal implementations      *}
{*                    of algorithmes from                          *}
{*              "Art of computer programming" by D.E.Knuth         *}
{*                   Copyright (c) by SoftWeyr ,1996    ^^ -! ;-)  *}
{*******************************************************************}
Unit swrandom;

interface

 Function NormRandom:Real;
 {Generates normally distributed random number, using
 Algorithm 3.4.1 P from D.E.Knuth, The art of computer programming, Vol 2}
 Function ExpRandom:Real;
 {Generates exponentially distributed random number, using
  logarithmic method, described in D.E.Knuth, The art... vol 2, 3.4.1}
 Type TSwapProc = procedure (i,j:Word);
 {Type for user-supplied swapping procedure}
Procedure Shuffle(N:Word;Swap:TSwapProc);
{Shuffles user's data structure of N elements, using user-supplied
 swap procedure, which swaps two elements, given by indices.
 Algorithm 3.4.2 P from Knuth, vol 2}
 Type TReadProc=procedure(Use:boolean);
 {type for user supplied procedure for skip data in input stream or
  include them into sample}
 Procedure Choose(n,total:longint;ReadOne:TReadProc);
 {Chooses n elements from total. If element is to be included, calls ReadOne
  with True parameter, otherwise with false. Algorithm 3.4.2 S}
function SentenceOfDay(var F:Text):String;
{Selects one random line from text file. Algorithm 3.4.2 R with buffer size
1}
implementation
function NormRandom:real;
{Algorithm 3.4.1 P from D.E.Knuth, The art of computer programming, Vol 2}
const buf:real=0;
      bufFilled:boolean=false;
var t,s,v1,v2:real;
begin
 if bufFilled then
  begin
   NormRandom:=buf;
   BufFilled:=False;
  end
 else
  begin
   repeat
    v1:=2*random-1;
    v2:=2*random-1;
    s:=sqr(v1)+sqr(v2);
  until s<1;
  t:=sqrt(-2*ln(s)/s);
  NormRandom:=v1*t;
  Buf:=v2*t;
  BufFilled:=True;
 end;
end;
function ExpRandom:real;
var W:Real;
begin
 repeat
  W:=Random;
 until W<>0;
 ExpRandom:=-ln(W);
end;
Procedure Shuffle(N:Word;Swap:TSwapProc);
{3.4.2 P}
var k,i,j:word;
begin
 for j:=N downto 2 do
  begin
   k:=random(j)+1;
   Swap(k,j);
  end;
end;
Procedure Choose(n,total:longint;ReadOne:TReadProc);
var m,t:longint;
begin
 t:=0;m:=0;
 repeat
 if round((Total-t)*random)<n-m then
 begin
  ReadOne(True);inc(m);
 end
 else
  ReadOne(False);
 inc(t);
 Until (m=n);
end;
function SentenceOfDay(var F:Text):String;
var I,U:Word;
    S:String;
begin
 U:=0;
 Reset(F);
 repeat
  I:=Random(65535);
  if i>U then
   begin
    Readln(F,S);
    U:=i;
   end else Readln(S);
  Until eof(F);
 SentenceOfDay:=S;
end;
end.